@@ -11,24 +11,19 @@
 # SPDX-License-Identifier: GPL-2.0-or-later
 from __future__ import annotations
 
-from collections.abc import MutableSequence, Sequence
-from gettext import gettext as _
-
-from . import util
-
-from . import encoding
-
-import re
-import sys
-import struct
-import signal
-
-from .crpatch import PatchRoot, Header, Hunk, HunkLine
-
 import curses
 import fcntl
+import re
+import signal
+import struct
+import sys
 import termios
+from collections.abc import MutableSequence, Sequence
+from gettext import gettext as _
+from textwrap import dedent
 
+from . import encoding, util
+from .crpatch import Header, Hunk, HunkLine, PatchRoot
 
 _origstdout = sys.__stdout__  # used by gethw()
 
@@ -43,7 +38,8 @@ def gethw() -> tuple[int, int]:
 
     """
     h, w = struct.unpack(
-        "hhhh", fcntl.ioctl(_origstdout, termios.TIOCGWINSZ, b"\0"*8))[0:2]
+        "hhhh", fcntl.ioctl(_origstdout, termios.TIOCGWINSZ, b"\0" * 8),
+    )[0:2]
     return h, w
 
 
@@ -57,6 +53,7 @@ def chunkselector(opts, headerlist, ui):
     # This is required for ncurses to display non-ASCII characters in default user
     # locale encoding correctly.  --immerrr
     import locale
+
     locale.setlocale(locale.LC_ALL, '')
 
     class DummyStdscr:
@@ -76,14 +73,21 @@ def chunkselector(opts, headerlist, ui):
     signal.signal(signal.SIGTSTP, f)
 
 
-_headermessages = {
+header_messages = {
     # {operation: text}
     'crecord': _('Select hunks to commit'),
     'cstage': _('Select hunks to stage'),
     'cunstage': _('Select hunks to keep'),
 }
 
-_confirmmessages = {
+main_operation_messages = {
+    # {operation: text}
+    'crecord': _('c: commit'),
+    'cstage': _('s: stage'),
+    'cunstage': None,  # TODO: not implemented!
+}
+
+confirm_messages = {
     'crecord': _('Are you sure you want to commit the selected changes [Yn]?'),
     'cstage': _('Are you sure you want to stage the selected changes [Yn]?'),
     'cunstage': _('Are you sure you want to unstage the unselected changes [Yn]?'),
@@ -143,9 +147,7 @@ class CursesChunkSelector:
         self.waslasttoggleallapplied = True
 
     def handlefirstlineevent(self):
-        """
-        Handle 'g' to navigate to the top most file in the ncurses window.
-        """
+        """Handle 'g' to navigate to the top most file in the ncurses window."""
         self.currentselecteditem = self.headerlist[0]
         currentitem = self.currentselecteditem
         # select the parent item recursively until we're at a header
@@ -159,9 +161,9 @@ class CursesChunkSelector:
         self.currentselecteditem = currentitem
 
     def handlelastlineevent(self):
-        """
-        Handle 'G' to navigate to the bottom most file/hunk/line depending
-        on the whether the fold is active or not.
+        """Handle 'G' to navigate to the bottom most file/hunk/line.
+
+        The behaviour depends on the whether the fold is active or not.
 
         If the bottom most file is folded, it navigates to that file and stops there.
         If the bottom most file is unfolded, it navigates to the bottom most hunk in
@@ -228,7 +230,6 @@ class CursesChunkSelector:
         a hunk is currently selected, then select the next hunk, if one exists,
         or if not, the next header if one exists.
         """
-        #self.startprintline += 1 #debug
         currentitem = self.currentselecteditem
 
         nextitem = currentitem.nextitem()
@@ -262,10 +263,7 @@ class CursesChunkSelector:
         self.recenterdisplayedarea()
 
     def rightarrowevent(self):
-        """
-        Select (if possible) the first of this item's child-items.
-
-        """
+        """Select (if possible) the first of this item's child-items."""
         currentitem = self.currentselecteditem
         nextitem = currentitem.firstchild()
 
@@ -281,11 +279,11 @@ class CursesChunkSelector:
         self.recenterdisplayedarea()
 
     def leftarrowevent(self):
-        """
-        If the current item can be folded (i.e. it is an unfolded header or
-        hunk), then fold it.  Otherwise try select (if possible) the parent
-        of this item.
+        """Select parent or fold.
 
+        If the current item can be folded (i.e. it is an unfolded header or
+        hunk), then fold it. Otherwise, try to select (if possible) the parent
+        of this item.
         """
         currentitem = self.currentselecteditem
 
@@ -335,7 +333,6 @@ class CursesChunkSelector:
         """Scroll the screen to fully show the currently-selected"""
         selstart = self.selecteditemstartline
         selend = self.selecteditemendline
-        #selnumlines = selend - selstart
         padstart = self.firstlineofpadtoprint
         padend = padstart + self.yscreensize - self.numstatuslines - 1
         # 'buffered' pad start/end values which scroll with a certain
@@ -392,13 +389,13 @@ class CursesChunkSelector:
                 hunkline.applied = item.applied
 
             siblingappliedstatus = [hnk.applied for hnk in item.header.hunks]
-            allsiblingsapplied = not (False in siblingappliedstatus)
-            nosiblingsapplied = not (True in siblingappliedstatus)
+            allsiblingsapplied = all(siblingappliedstatus)
+            nosiblingsapplied = not any(siblingappliedstatus)
 
             siblingspartialstatus = [hnk.partial for hnk in item.header.hunks]
-            somesiblingspartial = (True in siblingspartialstatus)
+            somesiblingspartial = any(siblingspartialstatus)
 
-            #cases where applied or partial should be removed from header
+            # cases where applied or partial should be removed from header
 
             # if no 'sibling' hunks are applied (including this hunk)
             if nosiblingsapplied:
@@ -407,13 +404,11 @@ class CursesChunkSelector:
                     item.header.partial = False
             else:  # some/all parent siblings are applied
                 item.header.applied = True
-                item.header.partial = (somesiblingspartial or
-                                       not allsiblingsapplied)
-
+                item.header.partial = somesiblingspartial or not allsiblingsapplied
         elif isinstance(item, HunkLine):
             siblingappliedstatus = [ln.applied for ln in item.hunk.changedlines]
-            allsiblingsapplied = not (False in siblingappliedstatus)
-            nosiblingsapplied = not (True in siblingappliedstatus)
+            allsiblingsapplied = all(siblingappliedstatus)
+            nosiblingsapplied = not any(siblingappliedstatus)
 
             # if no 'sibling' lines are applied
             if nosiblingsapplied:
@@ -426,14 +421,12 @@ class CursesChunkSelector:
                 item.hunk.applied = True
                 item.hunk.partial = True
 
-            parentsiblingsapplied = [hnk.applied for hnk
-                                     in item.hunk.header.hunks]
-            noparentsiblingsapplied = not (True in parentsiblingsapplied)
-            allparentsiblingsapplied = not (False in parentsiblingsapplied)
+            parentsiblingsapplied = [hnk.applied for hnk in item.hunk.header.hunks]
+            noparentsiblingsapplied = not any(parentsiblingsapplied)
+            allparentsiblingsapplied = all(parentsiblingsapplied)
 
-            parentsiblingspartial = [hnk.partial for hnk
-                                     in item.hunk.header.hunks]
-            someparentsiblingspartial = (True in parentsiblingspartial)
+            parentsiblingspartial = [hnk.partial for hnk in item.hunk.header.hunks]
+            someparentsiblingspartial = any(parentsiblingspartial)
 
             # if all parent hunks are not applied, un-apply header
             if noparentsiblingsapplied:
@@ -443,8 +436,7 @@ class CursesChunkSelector:
             # set the applied and partial status of the header if needed
             else:  # some/all parent siblings are applied
                 item.hunk.header.applied = True
-                item.hunk.header.partial = (someparentsiblingspartial or
-                                            not allparentsiblingsapplied)
+                item.hunk.header.partial = someparentsiblingspartial or not allparentsiblingsapplied
 
     def toggleall(self):
         """Toggle the applied flag of all items."""
@@ -477,7 +469,6 @@ class CursesChunkSelector:
         if isinstance(item, (Header, Hunk)):
             item.folded = not item.folded
 
-
     def alignstring(self, instr, window):
         """
         Add whitespace to the end of a string in order to make it fill
@@ -494,8 +485,19 @@ class CursesChunkSelector:
         numspaces = width - ((strwidth + xstart) % width)
         return instr + " " * numspaces
 
-    def printstring(self, window, text, fgcolor=None, bgcolor=None, pair=None,
-        pairname=None, attrlist=None, towin=True, align=True, showwhtspc=False):
+    def printstring(
+        self,
+        window,
+        text,
+        fgcolor=None,
+        bgcolor=None,
+        pair=None,
+        pairname=None,
+        attrlist=None,
+        towin=True,
+        align=True,
+        showwhtspc=False,
+    ):
         """
         Print the string, text, with the specified colors and attributes, to
         the specified curses window object.
@@ -520,7 +522,7 @@ class CursesChunkSelector:
         # Strip \n, and convert control characters to ^[char] representation
         text = re.sub(
             r'[\x00-\x08\x0a-\x1f]',
-            lambda m: '^' + chr(ord(m.group()) + 64), text.strip('\n')
+            lambda m: '^' + chr(ord(m.group()) + 64), text.strip('\n'),
         )
 
         if pair is not None:
@@ -588,10 +590,10 @@ class CursesChunkSelector:
         """-> [str]. return segments"""
         selected = self.currentselecteditem.applied
         segments = [
-            _headermessages[self.opts['operation']],
+            header_messages[self.opts['operation']],
             '-',
             _('[x]=selected **=collapsed'),
-            _('c: confirm'),
+            main_operation_messages[self.opts['operation']],
             _('q: abort'),
             _('arrow keys: move/expand/collapse'),
             _('space: deselect') if selected else _('space: select'),
@@ -643,10 +645,12 @@ class CursesChunkSelector:
         try:
             self.printitem()
             self.updatescroll()
-            self.chunkpad.refresh(self.firstlineofpadtoprint, 0,
-                                  self.numstatuslines, 0,
-                                  self.yscreensize - self.numstatuslines,
-                                  self.xscreensize)
+            self.chunkpad.refresh(
+                self.firstlineofpadtoprint, 0,
+                self.numstatuslines, 0,
+                self.yscreensize - self.numstatuslines,
+                self.xscreensize,
+            )
         except curses.error:
             pass
 
@@ -683,8 +687,9 @@ class CursesChunkSelector:
 
         return checkbox
 
-    def printheader(self, header: Header, selected=False, towin=True,
-                    ignorefolding=False):
+    def printheader(
+        self, header: Header, selected=False, towin=True, ignorefolding=False,
+    ):
         """
         Print the header to the pad.  If countLines is True, don't print
         anything, but just count the number of lines which would be printed.
@@ -696,11 +701,13 @@ class CursesChunkSelector:
 
         if chunkindex != 0 and not header.folded:
             # add separating line before headers
-            outstr += self.printstring(self.chunkpad, '_' * self.xscreensize,
-                                       towin=towin, align=False)
+            outstr += self.printstring(
+                self.chunkpad, "_" * self.xscreensize, towin=towin, align=False,
+            )
         # select color-pair based on if the header is selected
-        colorpair = self.getcolorpair(name=selected and "selected" or "normal",
-                                      attrlist=[curses.A_BOLD])
+        colorpair = self.getcolorpair(
+            name=selected and "selected" or "normal", attrlist=[curses.A_BOLD],
+        )
 
         # print out each line of the chunk, expanding it to screen width
 
@@ -712,42 +719,45 @@ class CursesChunkSelector:
             linestr = checkbox + textlist[0]
         else:
             linestr = checkbox + header.filename()
-        outstr += self.printstring(self.chunkpad, linestr, pair=colorpair,
-                                   towin=towin)
+        outstr += self.printstring(self.chunkpad, linestr, pair=colorpair, towin=towin)
         if not header.folded or ignorefolding:
             if len(textlist) > 1:
                 for line in textlist[1:]:
-                    linestr = " "*(indentnumchars + len(checkbox)) + line
-                    outstr += self.printstring(self.chunkpad, linestr,
-                                               pair=colorpair, towin=towin)
+                    linestr = " " * (indentnumchars + len(checkbox)) + line
+                    outstr += self.printstring(
+                        self.chunkpad, linestr, pair=colorpair, towin=towin,
+                    )
 
         return outstr
 
-    def printhunklinesbefore(self, hunk: Hunk, selected=False, towin=True,
-                             ignorefolding=False):
-        """includes start/end line indicator"""
+    def printhunklinesbefore(
+        self, hunk: Hunk, selected=False, towin=True, ignorefolding=False,
+    ):
+        """Print lines including the start/end line indicator."""
         outstr = ""
         # where hunk is in list of siblings
         hunkindex = hunk.header.hunks.index(hunk)
 
         if hunkindex != 0:
             # add separating line before headers
-            outstr += self.printstring(self.chunkpad, ' '*self.xscreensize,
-                                       towin=towin, align=False)
+            outstr += self.printstring(
+                self.chunkpad, " " * self.xscreensize, towin=towin, align=False,
+            )
 
-        colorpair = self.getcolorpair(name=selected and "selected" or "normal",
-                                      attrlist=[curses.A_BOLD])
+        colorpair = self.getcolorpair(
+            name=selected and "selected" or "normal", attrlist=[curses.A_BOLD],
+        )
 
         # print out from-to line with checkbox
         checkbox = self.getstatusprefixstring(hunk)
 
-        lineprefix = " "*self.hunkindentnumchars + checkbox
+        lineprefix = " " * self.hunkindentnumchars + checkbox
         frtoline = "   " + hunk.getfromtoline().decode("UTF-8", errors="hexreplace").strip("\n")
 
-        outstr += self.printstring(self.chunkpad, lineprefix, towin=towin,
-                                   align=False)  # add uncolored checkbox/indent
-        outstr += self.printstring(self.chunkpad, frtoline, pair=colorpair,
-                                   towin=towin)
+        outstr += self.printstring(
+            self.chunkpad, lineprefix, towin=towin, align=False,
+        )  # add uncolored checkbox/indent
+        outstr += self.printstring(self.chunkpad, frtoline, pair=colorpair, towin=towin)
 
         if hunk.folded and not ignorefolding:
             # skip remainder of output
@@ -755,7 +765,7 @@ class CursesChunkSelector:
 
         # print out lines of the chunk preceding changed-lines
         for line in hunk.before:
-            linestr = " "*(self.hunklineindentnumchars + len(checkbox)) + line.decode("UTF-8", errors="hexreplace")
+            linestr = " " * (self.hunklineindentnumchars + len(checkbox)) + line.decode("UTF-8", errors="hexreplace")
             outstr += self.printstring(self.chunkpad, linestr, towin=towin)
 
         return outstr
@@ -768,7 +778,7 @@ class CursesChunkSelector:
         # a bit superfluous, but to avoid hard-coding indent amount
         checkbox = self.getstatusprefixstring(hunk)
         for line in hunk.after:
-            linestr = " "*(self.hunklineindentnumchars + len(checkbox)) + line.decode("UTF-8", errors="hexreplace")
+            linestr = " " * (self.hunklineindentnumchars + len(checkbox)) + line.decode("UTF-8", errors="hexreplace")
             outstr += self.printstring(self.chunkpad, linestr, towin=towin)
 
         return outstr
@@ -789,17 +799,20 @@ class CursesChunkSelector:
         elif linestr.startswith("\\"):
             colorpair = self.getcolorpair(name="normal")
 
-        lineprefix = " "*self.hunklineindentnumchars + checkbox
-        outstr += self.printstring(self.chunkpad, lineprefix, towin=towin,
-                                   align=False)  # add uncolored checkbox/indent
-        outstr += self.printstring(self.chunkpad, linestr, pair=colorpair,
-                                   towin=towin, showwhtspc=True)
+        lineprefix = " " * self.hunklineindentnumchars + checkbox
+        outstr += self.printstring(
+            self.chunkpad, lineprefix, towin=towin, align=False,
+        )  # add uncolored checkbox/indent
+        outstr += self.printstring(
+            self.chunkpad, linestr, pair=colorpair, towin=towin, showwhtspc=True,
+        )
         return outstr
 
-    def printitem(self, item=None, ignorefolding=False, recursechildren=True,
-                  towin=True):
-        """
-        Use __printitem() to print the specified item.applied.
+    def printitem(
+        self, item=None, ignorefolding=False, recursechildren=True, towin=True,
+    ):
+        """Print the specified item.
+
         If item is not specified, then print the entire patch.
         (hiding folded elements, etc. -- see __printitem() docstring)
         """
@@ -810,7 +823,7 @@ class CursesChunkSelector:
 
         outstr = []
         self.__printitem(
-            item, ignorefolding, recursechildren, outstr, towin=towin
+            item, ignorefolding, recursechildren, outstr, towin=towin,
         )
         return ''.join(outstr)
 
@@ -824,14 +837,13 @@ class CursesChunkSelector:
         return y < miny or y > maxy
 
     def handleselection(self, item, recursechildren):
-        selected = (item is self.currentselecteditem)
+        selected = item is self.currentselecteditem
         if selected and recursechildren:
             # assumes line numbering starting from line 0
             self.selecteditemstartline = self.linesprintedtopadsofar
-            selecteditemlines = self.getnumlinesdisplayed(item,
-                                                          recursechildren=False)
-            self.selecteditemendline = (self.selecteditemstartline +
-                                        selecteditemlines - 1)
+            selecteditemlines = self.getnumlinesdisplayed(item, recursechildren=False)
+            self.selecteditemendline = self.selecteditemstartline + selecteditemlines - 1
+
         return selected
 
     def __printitem(
@@ -862,36 +874,42 @@ class CursesChunkSelector:
         if isinstance(item, PatchRoot):
             if recursechildren:
                 for hdr in item:
-                    self.__printitem(hdr, ignorefolding,
-                            recursechildren, outstr, towin)
+                    self.__printitem(hdr, ignorefolding, recursechildren, outstr, towin)
         # TODO: eliminate all isinstance() calls
         if isinstance(item, Header):
-            outstr.append(self.printheader(item, selected, towin=towin,
-                                       ignorefolding=ignorefolding))
+            outstr.append(
+                self.printheader(
+                    item, selected, towin=towin, ignorefolding=ignorefolding,
+                ),
+            )
             if recursechildren:
                 for hnk in item.hunks:
-                    self.__printitem(hnk, ignorefolding,
-                            recursechildren, outstr, towin)
-        elif (isinstance(item, Hunk) and
-              ((not item.header.folded) or ignorefolding)):
+                    self.__printitem(hnk, ignorefolding, recursechildren, outstr, towin)
+        elif isinstance(item, Hunk) and ((not item.header.folded) or ignorefolding):
             # print the hunk data which comes before the changed-lines
-            outstr.append(self.printhunklinesbefore(item, selected, towin=towin,
-                                                ignorefolding=ignorefolding))
+            outstr.append(
+                self.printhunklinesbefore(
+                    item, selected, towin=towin, ignorefolding=ignorefolding,
+                ),
+            )
             if recursechildren:
                 for line in item.changedlines:
-                    self.__printitem(line, ignorefolding,
-                            recursechildren, outstr, towin)
-                outstr.append(self.printhunklinesafter(item, towin=towin,
-                                                ignorefolding=ignorefolding))
-        elif (isinstance(item, HunkLine) and
-              ((not item.hunk.folded) or ignorefolding)):
-            outstr.append(self.printhunkchangedline(item, selected,
-                towin=towin))
+                    self.__printitem(
+                        line, ignorefolding, recursechildren, outstr, towin,
+                    )
+                outstr.append(
+                    self.printhunklinesafter(
+                        item, towin=towin, ignorefolding=ignorefolding,
+                    ),
+                )
+        elif isinstance(item, HunkLine) and ((not item.hunk.folded) or ignorefolding):
+            outstr.append(self.printhunkchangedline(item, selected, towin=towin))
 
         return outstr
 
-    def getnumlinesdisplayed(self, item=None, ignorefolding=False,
-                             recursechildren=True):
+    def getnumlinesdisplayed(
+        self, item=None, ignorefolding=False, recursechildren=True,
+    ):
         """
         Return the number of lines which would be displayed if the item were
         to be printed to the display.  The item will NOT be printed to the
@@ -902,8 +920,9 @@ class CursesChunkSelector:
 
         """
         # temporarily disable printing to windows by printstring
-        patchdisplaystring = self.printitem(item, ignorefolding,
-                                            recursechildren, towin=False)
+        patchdisplaystring = self.printitem(
+            item, ignorefolding, recursechildren, towin=False,
+        )
         numlines = len(patchdisplaystring) // self.xscreensize
         return numlines
 
@@ -919,8 +938,7 @@ class CursesChunkSelector:
         except curses.error:
             pass
 
-    def getcolorpair(self, fgcolor=None, bgcolor=None, name=None,
-                     attrlist=None):
+    def getcolorpair(self, fgcolor=None, bgcolor=None, name=None, attrlist=None):
         """
         Get a curses color pair, adding it to self.colorPairs if it is not
         already defined.  An optional string, name, can be passed as a shortcut
@@ -950,8 +968,9 @@ class CursesChunkSelector:
                 pairindex = len(self.colorpairs) + 1
                 if self.usecolor:
                     curses.init_pair(pairindex, fgcolor, bgcolor)
-                    colorpair = self.colorpairs[(fgcolor, bgcolor)] = (
-                        curses.color_pair(pairindex))
+                    colorpair = self.colorpairs[(fgcolor, bgcolor)] = curses.color_pair(
+                        pairindex,
+                    )
                     if name is not None:
                         self.colorpairnames[name] = curses.color_pair(pairindex)
                 else:
@@ -976,10 +995,6 @@ class CursesChunkSelector:
                     colorpair |= textattrib
         return colorpair
 
-    def initcolorpair(self, *args, **kwargs):
-        """Same as getcolorpair."""
-        self.getcolorpair(*args, **kwargs)
-
     def helpwindow(self):
         """Print a help window to the screen.  Exit after any keypress."""
         helptext = """            [press any key to return to the patch-display]
@@ -1011,8 +1026,9 @@ The following are valid keystrokes:
 
         helpwin = curses.newwin(self.yscreensize, 0, 0, 0)
         helplines = helptext.split("\n")
-        helplines = helplines + [" "]*(
-            self.yscreensize - self.numstatuslines - len(helplines) - 1)
+        helplines = helplines + [" "] * (
+            self.yscreensize - self.numstatuslines - len(helplines) - 1
+        )
         try:
             self.printstring(helpwin, helplines[0], pairname="legend")
             for line in helplines[1:]:
@@ -1027,7 +1043,6 @@ The following are valid keystrokes:
 
     def confirmationwindow(self, windowtext):
         """Display an informational window, then wait for and return a keypress."""
-
         lines = windowtext.split("\n")
         confirmwin = curses.newwin(len(lines), 0, 0, 0)
         try:
@@ -1049,35 +1064,36 @@ The following are valid keystrokes:
             return True
 
         if review:
-            confirmtext = (
-"""If you answer yes to the following, the your currently chosen patch chunks
-will be loaded into an editor.  You may modify the patch from the editor, and
-save the changes if you wish to change the patch.  Otherwise, you can just
-close the editor without saving to accept the current patch as-is.
-
-NOTE: don't add/remove lines unless you also modify the range information.
-      Failing to follow this rule will result in the commit aborting.
-
-Are you sure you want to review/edit and confirm the selected changes [Yn]?
-""")
+            confirmtext = dedent(
+                """
+                If you answer yes to the following, the your currently chosen patch chunks
+                will be loaded into an editor.  You may modify the patch from the editor, and
+                save the changes if you wish to change the patch.  Otherwise, you can just
+                close the editor without saving to accept the current patch as-is.
+                
+                NOTE: don't add/remove lines unless you also modify the range information.
+                      Failing to follow this rule will result in the commit aborting.
+                
+                Are you sure you want to review/edit and confirm the selected changes [Yn]?
+                """,
+            )
         else:
-            confirmtext = _confirmmessages[self.opts['operation']]
+            confirmtext = confirm_messages[self.opts['operation']]
 
         response = self.confirmationwindow(confirmtext)
 
         if response is None or len(response) == 0 or response == "\n":
             response = "y"
-        if response.lower().startswith("y"):
-            return True
-        else:
-            return False
+        return response.lower().startswith("y")
 
     def recenterdisplayedarea(self):
         """
-        once we scrolled with pg up pg down we can be pointing outside of the
-        display zone. we print the patch with towin=False to compute the
-        location of the selected item even though it is outside of the displayed
-        zone and then update the scroll.
+        Recenter the screen.
+
+        Once we scrolled with PgUp/PgDown, we can be pointing outside the
+        display zone. We print the patch with towin=False to compute the
+        location of the selected item even though it is outside the displayed
+        zone, and then update the scroll.
         """
         self.printitem(towin=False)
         self.updatescroll()
@@ -1088,31 +1104,25 @@ Are you sure you want to review/edit and confirm the selected changes [Yn]?
         When the amend flag is set, a commit will modify the most recent
         commit, instead of creating a new commit.  Otherwise, a
         new commit will be created (the normal commit behavior).
-
         """
         if not self.opts.get('amend'):
             self.opts['amend'] = True
-            msg = ("Amend option is turned on -- committing the currently "
-                   "selected changes will not create a new commit, but "
-                   "instead update the most recent commit.\n\n"
-                   "Press any key to continue.")
+            msg = (
+                "Amend option is turned on -- committing the currently "
+                "selected changes will not create a new commit, but "
+                "instead update the most recent commit.\n\n"
+                "Press any key to continue."
+            )
         else:
             self.opts['amend'] = False
-            msg = ("Amend option is turned off -- committing the currently "
-                   "selected changes will create a new commit.\n\n"
-                   "Press any key to continue.")
+            msg = (
+                "Amend option is turned off -- committing the currently "
+                "selected changes will create a new commit.\n\n"
+                "Press any key to continue."
+            )
 
         self.confirmationwindow(msg)
 
-    def emptypatch(self):
-        item = self.headerlist
-        if not item:
-            return True
-        for header in item:
-            if header.hunks:
-                return False
-        return True
-
     def handlekeypressed(self, keypressed):
         """
         Perform actions based on pressed keys.
@@ -1138,13 +1148,14 @@ Are you sure you want to review/edit and confirm the selected changes [Yn]?
         elif keypressed in ['a']:
             self.toggleamend()
         elif keypressed in ["c"]:
+            self.opts['confirm'] |= self.opts['operation'] != 'crecord'
             if self.confirmcommit():
-                self.opts['commit'] = True
+                self.opts['operation'] = 'crecord'
                 return True
         elif keypressed in ["s"]:
-            self.opts['commit'] = False
+            self.opts['confirm'] |= self.opts['operation'] != 'cstage'
             if self.confirmcommit():
-                self.opts['commit'] = False
+                self.opts['operation'] = 'cstage'
                 return True
         elif keypressed in ["r"]:
             if self.confirmcommit(review=True):
@@ -1176,16 +1187,15 @@ Are you sure you want to review/edit and confirm the selected changes [Yn]?
         return False
 
     def main(self, stdscr, opts):
-        """
-        Method to be wrapped by curses.wrapper() for selecting chunks.
+        """Configure and restore signal handler, and run the record UI.
 
+        Method to be wrapped by curses.wrapper() for selecting chunks.
         """
         self.opts = opts
 
         origsigwinch = sentinel = object()
         if util.safehasattr(signal, 'SIGWINCH'):
-             origsigwinch = signal.signal(signal.SIGWINCH,
-                                          self.sigwinchhandler)
+            origsigwinch = signal.signal(signal.SIGWINCH, self.sigwinchhandler)
         try:
             return self._main(stdscr)
         finally:
@@ -1193,6 +1203,7 @@ Are you sure you want to review/edit and confirm the selected changes [Yn]?
                 signal.signal(signal.SIGWINCH, origsigwinch)
 
     def _main(self, stdscr):
+        """Configure the display and run the event loop."""
         self.stdscr = stdscr
         # error during initialization, cannot be printed in the curses
         # interface, it should be printed by the calling code
@@ -1219,12 +1230,11 @@ Are you sure you want to review/edit and confirm the selected changes [Yn]?
 
         # available colors: black, blue, cyan, green, magenta, white, yellow
         # init_pair(color_id, foreground_color, background_color)
-        self.initcolorpair(None, None, name="normal")
-        self.initcolorpair(curses.COLOR_WHITE, curses.COLOR_MAGENTA,
-                           name="selected")
-        self.initcolorpair(curses.COLOR_RED, None, name="deletion")
-        self.initcolorpair(curses.COLOR_GREEN, None, name="addition")
-        self.initcolorpair(curses.COLOR_WHITE, curses.COLOR_BLUE, name="legend")
+        self.getcolorpair(None, None, name="normal")
+        self.getcolorpair(curses.COLOR_WHITE, curses.COLOR_MAGENTA, name="selected")
+        self.getcolorpair(curses.COLOR_RED, None, name="deletion")
+        self.getcolorpair(curses.COLOR_GREEN, None, name="addition")
+        self.getcolorpair(curses.COLOR_WHITE, curses.COLOR_BLUE, name="legend")
         # newwin([height, width,] begin_y, begin_x)
         self.statuswin = curses.newwin(self.numstatuslines, 0, 0, 0)
         self.statuswin.keypad(True)  # interpret arrow-key, etc. ESC sequences
@@ -1245,7 +1255,8 @@ Are you sure you want to review/edit and confirm the selected changes [Yn]?
             return
         # initialize selecteditemendline (initial start-line is 0)
         self.selecteditemendline = self.getnumlinesdisplayed(
-            self.currentselecteditem, recursechildren=False)
+            self.currentselecteditem, recursechildren=False,
+        )
 
         # option which enables/disables patch-review (in editor) step
         self.opts['crecord_reviewpatch'] = False
