@@ -10,14 +10,12 @@
 #
 # SPDX-License-Identifier: GPL-2.0-or-later
 
-# stuff related specifically to patch manipulation / parsing
-from gettext import gettext as _
-
 import io
 import re
 from codecs import register_error
-
-from typing import IO, Iterator, Optional, Sequence, Union
+from gettext import gettext as _
+from typing import IO, Optional, Union
+from collections.abc import Iterator, Sequence
 
 from .util import unwrap_filename
 
@@ -132,9 +130,11 @@ def scanpatch(fp: IO[bytes]):
                 lr.push(fromfile)
             yield 'file', header
         elif line.startswith(b' '):
-            yield 'context', scanwhile(line, lambda l: l[0] in b' \\')
+            yield 'context', scanwhile(line, lambda l: l[0] in b' ')
         elif line[0] in b'-+':
             yield 'hunk', scanwhile(line, lambda l: l[0] in b'-+\\')
+        elif line.startswith(b'* Unmerged path '):
+            continue
         else:
             m = lines_re.match(line)
             if m:
@@ -149,6 +149,7 @@ class PatchNode:
     """
 
     folded: bool
+    applied: bool
     # a patch this node belongs to
     patch: 'PatchRoot'
 
@@ -246,11 +247,9 @@ class PatchNode:
         prevsibling = self.prevsibling()
         if prevsibling is not None:
             prevsiblinglastchild = prevsibling.lastchild()
-            if ((prevsiblinglastchild is not None) and
-                    not prevsibling.folded):
+            if (prevsiblinglastchild is not None) and not prevsibling.folded:
                 prevsiblinglclc = prevsiblinglastchild.lastchild()
-                if ((prevsiblinglclc is not None) and
-                        not prevsiblinglastchild.folded):
+                if (prevsiblinglclc is not None) and not prevsiblinglastchild.folded:
                     return prevsiblinglclc
                 else:
                     return prevsiblinglastchild
@@ -275,6 +274,7 @@ class PatchNode:
 
 class Header(PatchNode):
     """Patch header"""
+
     diff_re = re.compile(b'diff --git (?P<fromfile>(?P<aq>")?a/.*(?(aq)"|)) (?P<tofile>(?P<bq>")?b/.*(?(bq)"|))$')
     allhunks_re = re.compile(b'(?:GIT binary patch|new file|deleted file) ')
     pretty_re = re.compile(b'(?:new file|deleted file) ')
@@ -299,14 +299,11 @@ class Header(PatchNode):
         self._changetype = None
 
     def binary(self):
-        """
-        Return True if the file represented by the header is a binary file.
-        Otherwise return False.
-
-        """
+        """Return True if the file represented by the header is a binary file."""
         return any(h.startswith(b'GIT binary patch') for h in self.header)
 
     def pretty(self, fp: IO[str]):
+        """Pretty-print the header into a stream"""
         for h in self.header:
             if h.startswith(b'GIT binary patch'):
                 fp.write(_('this modifies a binary file (all or nothing)\n'))
@@ -317,9 +314,13 @@ class Header(PatchNode):
                     fp.write(_('this is a binary file\n'))
                 break
             if h.startswith(b'---'):
-                fp.write(_('%d hunks, %d lines changed\n') %
-                         (len(self.hunks),
-                          sum([max(h.added, h.removed) for h in self.hunks])))
+                fp.write(
+                    _('%d hunks, %d lines changed\n') %
+                    (
+                        len(self.hunks),
+                        sum(max(h.added, h.removed) for h in self.hunks),
+                    ),
+                )
                 break
             fp.write(h.decode("UTF-8", errors="hexreplace"))
 
@@ -357,9 +358,11 @@ class Header(PatchNode):
         return (files[1] or files[0]).decode("UTF-8", errors="hexreplace")
 
     def __repr__(self) -> str:
-        return '<header %s>' % (' '.join(
-            repr(x) for x in self.files()
-        ))
+        return '<header %s>' % (
+            ' '.join(
+                repr(x) for x in self.files()
+            )
+        )
 
     def special(self) -> bool:
         return any(self.special_re.match(h) for h in self.header)
@@ -427,7 +430,17 @@ class Header(PatchNode):
 class HunkLine(PatchNode):
     """Represents a changed line in a hunk"""
 
-    def __init__(self, linetext: bytes, hunk):
+    DELETE = b'-'
+    INSERT = b'+'
+    CONTEXT = b' '
+    HUNK = b'@'
+    NOEOL = b'\\'
+
+    linetext: bytes
+    hunk: 'Hunk'
+    offset: int
+
+    def __init__(self, linetext: bytes, hunk: 'Hunk'):
         self.linetext = linetext
         self.applied = True
         # the parent hunk to which this line belongs
@@ -435,9 +448,11 @@ class HunkLine(PatchNode):
         # folding lines currently is not used/needed, but this flag is needed
         # in the prevItem method.
         self.folded = False
+        # at this moment, the offset of the line in the hunk isn’t known yet
+        self.offset = 0
 
     def __bytes__(self):
-        if self.applied:
+        if self.applied or self.diffop == HunkLine.NOEOL:
             return self.linetext
         else:
             return b' ' + self.linetext[1:]
@@ -446,6 +461,14 @@ class HunkLine(PatchNode):
     def diffop(self):
         return self.linetext[0:1]
 
+    def __repr__(self):
+        return "<hunkline/%c/%d %s %s>" % (
+            self.linetext[0],
+            self.offset,
+            '[x]' if self.applied else '[ ]',
+            self.linetext[1:10],
+        )
+
     def __str__(self) -> str:
         return self.prettystr()
 
@@ -462,7 +485,7 @@ class HunkLine(PatchNode):
         else:
             return None
 
-    def prevsibling(self):
+    def prevsibling(self) -> Optional['HunkLine']:
         """Return the previous line in the hunk"""
         indexofthisline = self.hunk.changedlines.index(self)
         if indexofthisline > 0:
@@ -471,7 +494,7 @@ class HunkLine(PatchNode):
         else:
             return None
 
-    def parentitem(self):
+    def parentitem(self) -> 'Hunk':
         """Return the parent to the current item"""
         return self.hunk
 
@@ -488,6 +511,7 @@ class HunkLine(PatchNode):
 
 class Hunk(PatchNode):
     """ui patch hunk, wraps a hunk and keeps track of ui behavior """
+
     maxcontext = 3
     header: Header
     fromline: int
@@ -498,17 +522,18 @@ class Hunk(PatchNode):
     changedlines: Sequence[HunkLine]
 
     def __init__(
-            self,
-            header: Header,
-            fromline: int,
-            toline: int,
-            proc: bytes,
-            before: Sequence[bytes],
-            hunklines: Sequence[bytes],
-            after: Sequence[bytes]
+        self,
+        header: Header,
+        fromline: int,
+        toline: int,
+        proc: bytes,
+        before: Sequence[bytes],
+        hunklines: Sequence[bytes],
+        after: Sequence[bytes],
     ):
         def trimcontext(number, lines):
             delta = len(lines) - self.maxcontext
+            # TODO: why is this disabled?
             if False and delta > 0:
                 return number + delta, lines[:self.maxcontext]
             return number, lines
@@ -519,6 +544,7 @@ class Hunk(PatchNode):
         self.proc = proc
         self.changedlines = [HunkLine(line, self) for line in hunklines]
         self.added, self.removed = self.countchanges()
+        self.countoffsets()
         # used at end for detecting how many removed lines were un-applied
         self.originalremoved = self.removed
 
@@ -574,18 +600,40 @@ class Hunk(PatchNode):
 
     def countchanges(self) -> tuple[int, int]:
         """changedlines -> (n+,n-)"""
-        add = len([line for line in self.changedlines if line.applied
-                   and line.diffop == b'+'])
-        rem = len([line for line in self.changedlines if line.applied
-                   and line.diffop == b'-'])
+        add = len(
+            [line for line in self.changedlines if line.applied and line.diffop == HunkLine.INSERT],
+        )
+        rem = len(
+            [line for line in self.changedlines if line.applied and line.diffop == HunkLine.DELETE],
+        )
         return add, rem
 
+    def countoffsets(self):
+        fromline = 0
+        toline = 0
+        deletes = False
+        for line in self.changedlines:
+            if line.diffop == HunkLine.INSERT:
+                fromline += 1
+                line.offset = fromline
+            if line.diffop == HunkLine.DELETE:
+                deletes = True
+                toline += 1
+                line.offset = toline
+            if line.diffop == HunkLine.NOEOL:
+                if not deletes:
+                    line.offset = fromtoline
+                else:
+                    line.offset = toline
+
     def getfromtoline(self):
         """Calculate the number of removed lines converted to context lines"""
         removedconvertedtocontext = self.originalremoved - self.removed
 
-        contextlen = (len(self.before) + len(self.after) +
-                      removedconvertedtocontext)
+        contextlen = (
+            len(self.before) + len(self.after) +
+            removedconvertedtocontext
+        )
         if self.after and self.after[-1] == b'\\ No newline at end of file\n':
             contextlen -= 1
         fromlen = contextlen + self.removed
@@ -603,24 +651,45 @@ class Hunk(PatchNode):
         if tolen == 0 and toline > 0:
             toline -= 1
 
-        fromtoline = b'@@ -%d,%d +%d,%d @@%b\n' % (
-            fromline, fromlen, toline, tolen,
-            self.proc and (b' ' + self.proc))
+        fromtoline = b"@@ -%d,%d +%d,%d @@%b\n" % (
+            fromline,
+            fromlen,
+            toline,
+            tolen,
+            self.proc and (b" " + self.proc),
+        )
 
         return fromtoline
 
-    def write(self, fp: IO[bytes]) -> None:
+    def iterlines(self, all: bool = False):
         # updated self.added/removed, which are used by getfromtoline()
         self.added, self.removed = self.countchanges()
-        fp.write(self.getfromtoline())
-        fp.write(b''.join(self.before))
+        yield HunkLine.HUNK, self.getfromtoline()
+        for line in self.before:
+            yield HunkLine.CONTEXT, line
 
-        # add the following to the list: (1) all applied lines, and
-        # (2) all unapplied removal lines (convert these to context lines)
-        for changedline in self.changedlines:
-            fp.write(bytes(changedline))
+        # Include these lines:
+        # (1) all applied lines
+        # (2) all unapplied removal lines (converted context lines)
+        # Lines are sorted by their offset in the original hunk,
+        # with removals coming first.
+        for hunkline in sorted(
+            self.changedlines,
+            key=lambda line: (
+                line.offset, line.diffop == HunkLine.INSERT,
+            ),
+        ):
+            if hunkline.diffop == HunkLine.INSERT and not (hunkline.applied or all):
+                continue
 
-        fp.write(b''.join(self.after))
+            yield hunkline.diffop, bytes(hunkline)
+
+        for line in self.after:
+            yield HunkLine.CONTEXT, line
+
+    def write(self, fp: IO[bytes]) -> None:
+        for _, line in self.iterlines():
+            fp.write(line)
 
     def reversehunks(self) -> 'Hunk':
         r"""Make the hunk apply in the other direction.
@@ -642,10 +711,21 @@ class Hunk(PatchNode):
          4
          5
         """
-        m = {b'+': b'-', b'-': b'+', b'\\': b'\\'}
-        hunklines = [b'%s%s' % (m[line.linetext[0:1]], line.linetext[1:])
-                     for line in self.changedlines if line.applied]
-        return Hunk(self.header, self.fromline, self.toline, self.proc, self.before, hunklines, self.after)
+        m = {b"+": b"-", b"-": b"+", b"\\": b"\\"}
+        hunklines = [
+            b"%s%s" % (m[line.linetext[0:1]], line.linetext[1:])
+            for line in self.changedlines
+            if line.applied
+        ]
+        return Hunk(
+            self.header,
+            self.fromline,
+            self.toline,
+            self.proc,
+            self.before,
+            hunklines,
+            self.after,
+        )
 
     def files(self) -> list[Optional[bytes]]:
         return self.header.files()
@@ -743,6 +823,32 @@ def parsepatch(fp: IO[bytes]) -> PatchRoot:
     @@ -8,0 +10,1 @@
     +9
 
+    Parsing a patch changing the EOL at the EOF:
+    >>> rawpatch = b'''diff --git a/test b/test
+    ... --- a/test
+    ... +++ b/test
+    ... @@ -11,5 +11,5 @@
+    ...  foo
+    ...  bar
+    ...  baz
+    ... -qoox
+    ... -qooox
+    ... \\ No newline at end of file
+    ... +quux
+    ... +quuux'''
+    >>> fp = io.BytesIO(rawpatch)
+    >>> headers = parsepatch(fp)
+    >>> print(headers[0].hunks[0])
+    @@ -11,5 +11,5 @@
+     foo
+     bar
+     baz
+    -qoox
+    +quux
+    -qooox
+    \ No newline at end of file
+    +quuux
+
     It is possible to handle non-UTF-8 patches:
     >>> rawpatch = b'''diff --git a/test b/test
     ... --- /dev/null
@@ -826,8 +932,15 @@ def parsepatch(fp: IO[bytes]) -> PatchRoot:
             next hunk we parse.
 
             """
-            h = Hunk(self.header, self.fromline, self.toline, self.proc,
-                     self.before, self.hunk, self.context)
+            h = Hunk(
+                self.header,
+                self.fromline,
+                self.toline,
+                self.proc,
+                self.before,
+                self.hunk,
+                self.context,
+            )
             self.header.hunks.append(h)
             self.headers.append(h)
             self.fromline += len(self.before) + h.removed + len(self.context)
@@ -844,7 +957,7 @@ def parsepatch(fp: IO[bytes]) -> PatchRoot:
             Also, if an unprocessed set of changelines was previously
             encountered, this is the condition for creating a complete
             hunk object.  In this case, we create and add a new hunk object to
-            the most recent header object, and to self.strem. 
+            the most recent header object, and to self.strem.
 
             """
             self.context = context
@@ -872,7 +985,7 @@ def parsepatch(fp: IO[bytes]) -> PatchRoot:
             filename the header applies to.  Add the header to self.headers.
 
             """
-            # if there are any lines in the unchanged-lines buffer, create a 
+            # if there are any lines in the unchanged-lines buffer, create a
             # new hunk using them, and add it to the last header.
             if self.hunk:
                 self.add_new_hunk()
@@ -883,7 +996,7 @@ def parsepatch(fp: IO[bytes]) -> PatchRoot:
             self.header = h
 
         def finished(self):
-            # if there are any lines in the unchanged-lines buffer, create a 
+            # if there are any lines in the unchanged-lines buffer, create a
             # new hunk using them, and add it to the last header.
             if self.hunk:
                 self.add_new_hunk()
@@ -891,18 +1004,26 @@ def parsepatch(fp: IO[bytes]) -> PatchRoot:
             return self.headers
 
         transitions = {
-            'file': {'context': addcontext,
-                     'file': newfile,
-                     'hunk': addhunk,
-                     'range': addrange},
-            'context': {'file': newfile,
-                        'hunk': addhunk,
-                        'range': addrange},
-            'hunk': {'context': addcontext,
-                     'file': newfile,
-                     'range': addrange},
-            'range': {'context': addcontext,
-                      'hunk': addhunk},
+            'file': {
+                'context': addcontext,
+                'file': newfile,
+                'hunk': addhunk,
+                'range': addrange,
+            },
+            'context': {
+                'file': newfile,
+                'hunk': addhunk,
+                'range': addrange,
+            },
+            'hunk': {
+                'context': addcontext,
+                'file': newfile,
+                'range': addrange,
+            },
+            'range': {
+                'context': addcontext,
+                'hunk': addhunk,
+            },
         }
 
     p = Parser()
@@ -913,8 +1034,10 @@ def parsepatch(fp: IO[bytes]) -> PatchRoot:
         try:
             p.transitions[state][newstate](p, data)
         except KeyError:
-            raise PatchError('unhandled transition: %s -> %s' %
-                             (state, newstate))
+            raise PatchError(
+                'unhandled transition: %s -> %s' %
+                (state, newstate),
+            )
         state = newstate
     return PatchRoot(p.finished())
 
@@ -922,7 +1045,8 @@ def parsepatch(fp: IO[bytes]) -> PatchRoot:
 def filterpatch(opts, patch: PatchRoot, chunkselector, ui):
     r"""Interactively filter patch chunks into applied-only chunks
 
-    >>> rawpatch = b'''diff --git a/dir/file.c b/dir/file.c
+    >>> rawpatch = b'''* Unmerged path .gitignore
+    ... diff --git a/dir/file.c b/dir/file.c
     ... index e548702cb275..28208f7ff2ac 100644
     ... --- a/dir/file.c
     ... +++ b/dir/file.c
@@ -945,15 +1069,22 @@ def filterpatch(opts, patch: PatchRoot, chunkselector, ui):
     ...  16
     ...  17
     ... '''
+    >>> from functools import partial
+    >>> def hunk_selector(selections: Sequence[bool], opts, headers, ui):
+    ...     for i, hunk in enumerate(headers[0].hunks):
+    ...         hunk.applied = selections[i]
+    ...
+    >>> def line_selector(selections: Sequence[bool], opts, headers, ui):
+    ...     for i, hunkline in enumerate(headers[0].hunks[0].changedlines):
+    ...         hunkline.applied = selections[i]
+    ...
     >>> patch = parsepatch(io.BytesIO(rawpatch))
     >>> patch
     [<header b'dir/file.c' b'dir/file.c'>,
      <hunk b'dir/file.c'@1684>,
      <hunk b'dir/file.c'@1692>]
-    >>> def selector(opts, headers, ui):
-    ...     headers[0].hunks[0].applied = False
-    ... 
-    >>> applied = filterpatch(None, patch, selector, None)
+    >>> selections = [False, True]
+    >>> applied = filterpatch(None, patch, partial(hunk_selector, selections), None)
     >>> applied
     [<header b'dir/file.c' b'dir/file.c'>,
      <hunk b'dir/file.c'@1692>]
@@ -978,10 +1109,11 @@ def filterpatch(opts, patch: PatchRoot, chunkselector, ui):
 
     applied_hunks = PatchRoot([])
     for header in patch.headers:
-        if (header.applied and
-                (header.special() or header.binary() or len([
-                    h for h in header.hunks if h.applied
-                ]) > 0)):
+        if header.applied and (
+            header.special()
+            or header.binary()
+            or len([h for h in header.hunks if h.applied]) > 0
+        ):
             applied_hunks.append(header)
             fixoffset = 0
             for hunk in header.hunks:
